#include <assert.h>
#include <math.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#include "timing.h"

#define RESTRICT __restrict__
#define FUNC_PREFIX __global__
#include <cuda_runtime.h>

//kernels
{% for include in includes %}
#include "{{ include }}.h"
{% endfor %}

int main(int argc, char **argv)
{
    if(argc < 2) {
        printf("Usage: %s <n_repeat>\n", argv[0]);
        return -1;
    }
    int n_repeat = atoi(argv[1]);
    {% for kernel in kernels %}

    { // Kernel: {{kernel.name}}
        {% for field_name, dataType, elements in kernel.fields %}
          {{dataType}} *{{field_name}};
          cudaMalloc(&{{field_name}}, {{elements}}*sizeof({{dataType}}));
          cudaMemset({{field_name}},     0.23, {{elements}});
        {% endfor %}

        {% for constantName, dataType in kernel.constants %}
        // Constant {{constantName}}
        {{dataType}} {{constantName}};
        {{constantName}} = 0.23;
        {% endfor %}

        dim3 blocks({{kernel.blocks[0]}}, {{kernel.blocks[1]}}, {{kernel.blocks[2]}});
        dim3 grid({{kernel.grid[0]}}, {{kernel.grid[1]}}, {{kernel.grid[2]}});

        for(int warmup = 1; warmup >= 0; --warmup) {
            int repeat = 2;
            if(warmup == 0) {
                repeat = n_repeat;
            }

            {% if timing %}
            double wcStartTime, cpuStartTime, wcEndTime, cpuEndTime;
            timing(&wcStartTime, &cpuStartTime);
            {% endif %}

            for (; repeat > 0; --repeat)
            {
                {{kernel.name}}<<<grid, blocks>>>({{kernel.call_argument_list}});
            }

            {% if timing %}
            timing(&wcEndTime, &cpuEndTime);

            if( warmup == 0)
                printf("%s\t%e\n", "{{kernel.name}}",(wcEndTime - wcStartTime) / n_repeat );
            {% endif %}
        }
        {% for field_name, dataType, elements in kernel.fields %}
        cudaFree({{field_name}});
        {% endfor %}
    }
    {% endfor %}
}
